#include "mbed.h"
#include "RESTful.h"

//RESTful
RESTful::RESTful(PinName tx, PinName rx, PinName reset, int baud, int timeout)
{
		ark = new ESP8266(tx, rx, reset, baud, timeout);	
		gm = new TCPSocketConnection();
}

char* RESTful::generateGETRequest(char* HTTPRequest_Buffer, char* API_GET_MSG, char* Argument_String)
{
		//Clear the HTTPRequest buffer
		sprintf(HTTPRequest_Buffer,"%s","");

		strcat(HTTPRequest_Buffer,API_GET_MSG);
		strcat(HTTPRequest_Buffer,Argument_String);
		strcat(HTTPRequest_Buffer,"\r\n\r\n");
    return HTTPRequest_Buffer;
}

bool RESTful::HardwareInit()
{
		if(ark->init()) {
      //hardware intialized
			return true;
		}else{
			//hardware not intialized
			return false;
		}
}

bool RESTful::APConnection(char* SSID, char* Password)
{
    if(!ark->connect(SSID,Password)) {
			//conenction fail
			return false;
		}
    if(ark->is_connected()) {
			 //connection confirmed
			 return true;
			}else{
				//sonething went wrong with connection. not connected
				return false;
		}
}

const char* RESTful::getIPAddress()
{
		return ark->getIPAddress();
}

bool RESTful::HostConnection(char* Host, int Port)
{
    if(gm->connect(Host,Port)==0) {
				//Host connected
				return true;
			}else{
				//conenction fail
				return false;
			}
}

int RESTful::connectToAP(char* SSID, char* Password, int Retry_Times)
{
	int retry_counter = 0;
	Wifi_Error err = ERROR_NONE_ERROR;

	//HardwareInit
	while(1){
		if(HardwareInit()){
			//success
			retry_counter =0;
			break;
		}else{
			//failed
			if(retry_counter<Retry_Times){
			retry_counter++;
			}else{
				//failed 3 times
				//retry_counter=0;
				
				//return error code here
				err = ERROR_HardwareInit;
				return err;
			}
		}
	}
	
	//APConnection
	while(1){
		if(APConnection(SSID, Password)){
			//success
			retry_counter =0;
			break;
		}else{
			//failed
			if(retry_counter<Retry_Times){
			retry_counter++;
			}else{
				//failed 3 times
				//retry_counter=0;
				
				//return error code here
				err = ERROR_APConnection;
				return err;
			}
		}
	}
	//no error
	return err;
}

int RESTful::sendHTTPRequest(char* HTTPRequest, char* HTTPResponce, int HTTPResponce_Size, char* Host, int Port, int Retry_Times)
{
	int retry_counter = 0;
	Wifi_Error err = ERROR_NONE_ERROR;

	//HostConnection
	while(1){
		if(HostConnection(Host, Port)){
			//success
			retry_counter =0;
			break;
		}else{
			//failed
			if(retry_counter<Retry_Times){
			retry_counter++;
			}else{
				//failed 3 times
				//retry_counter=0;
				
				//return error code here
				err = ERROR_HostConnection;
				return err;
			}
		}
	}

	//sendGETRequest
	while(1){	
		//sending GET request
    if(gm->send_all(HTTPRequest,strlen(HTTPRequest))>0) {
       //reading responce of GET request!
       int resBytes = gm->receive_all(HTTPResponce,HTTPResponce_Size);
       if(resBytes > 0) {
          //Responce Received!
					retry_counter =0;
					err = Responce_Received;
					return err;
          } else if(resBytes==0) {
                    //Empty Responce!
										retry_counter =0;
										err = Empty_Responce;
										return err;
          } else {
										//Resopnce NOT Received!
										//failed
										if(retry_counter<Retry_Times){
											retry_counter++;
											}else{
														//failed 3 times
														//return error code here
														err = ERROR_Resopnce_NOT_Received;
														return err;
														}
								}
       } else {
           //GET request failed!
						//failed
					if(retry_counter<Retry_Times){
						retry_counter++;
					}else{
						//failed 3 times
						//return error code here
						err = ERROR_GET_Request_Failed;
						return err;
					}							
         }			
	}
}

bool RESTful::GetWifiCfg(char* SSID, char* Password, int Maximum_Response_Length)
{
//SSID and Password are stored in the file wifi.cfg. The physical location is the Flash IC on the ESP8266 module
//wifi.cfg format: 
//First line: SSID
//Second line: Password
	
// Wait for SSID and Password from ESP8266
Timer timer;
timer.start();
    
    while (true) {
			//Reset the module
      ark->reset();
			wait_ms(600);
			
				//Get SSID
				int resp_len = Maximum_Response_Length;
        if (!(ark->command("file.open('wifi.cfg');") &&
              ark->command("ssid=file.readline();") &&
							ark->command("print(ssid)") &&
              ark->execute(SSID, &resp_len)))
            return false;
        
        SSID[resp_len] = 0;
				
				//Get Password
				resp_len = Maximum_Response_Length;
				if (!(ark->command("password=file.readline();") &&
							ark->command("print(password)") &&
							ark->execute(Password, &resp_len)))
            return false;
        
        Password[resp_len] = 0;
        
				//Close File
				if (!(ark->command("file.close();") &&
							ark->execute()))
            return false;
				
        if (strcmp(SSID, "nil") != 0 && strcmp(Password, "nil") != 0)
            return true;
        
        if (timer.read_ms() > 500)
            return false;
    }
}	

bool RESTful::WifiSetup()
{
	/*
	LUA Code
	=====================================================================================================================
	
print("WIFI Setup")
wifi.setmode(wifi.SOFTAP)
print("The ESP8266 WiFi mode is: " .. wifi.getmode())
cfg={}
cfg.ssid="SENSOR_AP"
cfg.pwd="password"
if cfg.ssid and cfg.pwd then
print("ESP8266 SSID is " .. cfg.ssid .. " and PASSWORD is: " .. cfg.pwd)
end


-- Now you should see an SSID wireless router named ESP_STATION when you scan for available WIFI networks
-- Lets connect to the module from a computer of mobile device. So, find the SSID and connect using the password selected
wifi.ap.config(cfg)
ap_mac = wifi.ap.getmac()

-- create a server on port 80 and wait for a connection, when a connection is coming in function c will be executed
sv=net.createServer(net.TCP,30)
sv:listen(80,function(c)
c:on("receive", function(c, pl)

-- print the payload pl received from the connection
print(pl)
print(string.len(pl))

-- wait until SSID comes back and parse the SSID and the password
print(string.match(pl,"GET"))
ssid_start,ssid_end=string.find(pl,"SSID=")
if ssid_start and ssid_end then
amper1_start, amper1_end =string.find(pl,"&", ssid_end+1)
if amper1_start and amper1_end then
http_start, http_end =string.find(pl,"HTTP/1.1", ssid_end+1)
if http_start and http_end then
ssid=string.sub(pl,ssid_end+1, amper1_start-1)
password=string.sub(pl,amper1_end+10, http_start-2)
c:send("<h1>SSID and Password have been saved!</h1> ")
print("ESP8266 saved SSID: " .. ssid .. " with PASSWORD: " .. password)
if ssid and password then
sv:close()
-- save the ssid and password into wifi.cfg file
file.remove("wifi.cfg")
file.open("wifi.cfg", "w")
file.writeline(ssid)
file.writeline(password)
file.close()


end
end
end
end
-- this is the web page that requests the SSID and password from the user
c:send("<!DOCTYPE html> ")
c:send("<html lang='en'> ")
c:send("<body> ")
c:send("<h1>Wifi Setup</h1> ")
mac_mess1 = "The module MAC address is: " .. ap_mac
mac_mess2 = "You will need this MAC address to find the IP address of the module, please take note of it."

c:send("<h2>" .. mac_mess1 .. "</h2> ")
c:send("<h2>" .. mac_mess2 .. "</h2> ")
c:send("<h2>Enter SSID and Password for your WIFI router</h2> ")
c:send("</form> </body> </html>")
c:send("<form action='' method='GET'>")
c:send("SSID:")
c:send("<input type=text name='SSID' value='' maxlength='100' />")
c:send("<br />")
c:send("Password:")
c:send("<input type='text' name='Password' value='' maxlength='100' />")
c:send("<input type='submit' value='Submit' />")
end)
end)

	*/
	
	
//Reset the module
ark->reset();
wait_ms(600);

//Enter Setup Process, the process will let the user to enter the SSID and Password
if (!(ark->command("dofile(\"setup.lua\")") &&
      ark->execute()))
    return false;
        
return true;

}
