#include "mbed.h"
#include "PMS3003.h"

//PMS3003 with external power on/off design, PMS3003_EN
PMS3003::PMS3003(PinName USART_TX, PinName USART_RX, PinName PMS3003_RESET, PinName PMS3003_SET, PinName PMS3003_EN)
{
		USART = new Serial(USART_TX, USART_RX);
		USART->baud(9600);
		RESET = new DigitalOut(PMS3003_RESET);
		RESET->write(1); //Low level for reset
		SET = new DigitalOut(PMS3003_SET);
		SET->write(0); //High level for normal mode, low level for entering low power mode (Fan stopping)
		EN = new DigitalOut(PMS3003_EN);
		EN->write(0); //High level for power on, low level for power off (both logic part and fan sotpping)
		External_EN = true; //has external EN design
		is_normal = false; //init the sensor status to not normal, because it has not been inited
}

//PMS3003 without external power on/off design
PMS3003::PMS3003(PinName USART_TX, PinName USART_RX, PinName PMS3003_RESET, PinName PMS3003_SET)
{
		USART = new Serial(USART_TX, USART_RX);
		USART->baud(9600);
		RESET = new DigitalOut(PMS3003_RESET);
		RESET->write(1); //Low level for reset
		SET = new DigitalOut(PMS3003_SET);
		SET->write(0); //High level for normal mode, low level for entering low power mode (Fan stopping)
		External_EN = false; //without external EN design
		is_normal = false; //init the sensor status to not normal, because it has not been inited
}

void PMS3003::readData(){
	//PMS3003::pc->putc(USART->getc());
	char newdata;
	int i;
	unsigned short calculated_checksum =0; 
	unsigned short checksum =0;
	newdata = USART->getc();
	
	//shifting
	for(i = Shift_Register_Size; i>0; i--)
	{
		Shift_Register[i] = Shift_Register[i-1];
	}
	//Inserting
	Shift_Register[0] = newdata;
	
	//Check if a full frame have been received
	if(Shift_Register[Shift_Register_Size-1] == 0x42 && Shift_Register[Shift_Register_Size-2] == 0x4d)
	{
		//Calculate Checksum
		for(i = Shift_Register_Size-1; i>=2; i--)
		{
			calculated_checksum += Shift_Register[i];
		}
		checksum = (Shift_Register[1]<<8)|Shift_Register[0];
		
		if(calculated_checksum != checksum){
			is_normal = false; //set the sensor status to not normal, because the Corrupted Data has been received
			return; //Data corruption
		}
		
		is_normal = true; //set the sensor status to normal, because the Correct Data has been received
		
		//Get PM Data
		PM_Data.CF1PM1     = (Shift_Register[19]<<8)|Shift_Register[18];
		PM_Data.CF1PM2dot5 = (Shift_Register[17]<<8)|Shift_Register[16];
		PM_Data.CF1PM10    = (Shift_Register[15]<<8)|Shift_Register[14];
		PM_Data.PM1        = (Shift_Register[13]<<8)|Shift_Register[12];
		PM_Data.PM2dot5    = (Shift_Register[11]<<8)|Shift_Register[10];
		PM_Data.PM10       = (Shift_Register[9]<<8)|Shift_Register[8];
		
		/*
		//Printing data to serial port, for debug
		for(i = Shift_Register_Size-1; i>=0; i--)
		{
			//Shift_Register[i] = Shift_Register[i-1];
			//PMS3003::pc->putc(Shift_Register[i]);
			//PMS3003::pc->printf("PM1 (CF1): %d ug/m3	PM2.5 (CF1): %d ug/m3	PM10 (CF1): %d ug/m3 \r\n", PM_Data.CF1PM1, PM_Data.CF1PM2dot5, PM_Data.CF1PM10);
			 PMS3003::pc->printf("PM1: %d ug/m3		PM2.5: %d ug/m3		PM10: %d ug/m3 \r\n", PM_Data.PM1, PM_Data.PM2dot5, PM_Data.PM10);
		}
		*/
	}
}

void PMS3003::init() {
	//pc = new Serial(PC_10, PC_11); // tx, rx USART3, for debug
	USART->attach(this, &PMS3003::readData);
}

unsigned short PMS3003::getCF1PM1(){
	return PM_Data.CF1PM1;
}

unsigned short PMS3003::getCF1PM2dot5(){
	return PM_Data.CF1PM2dot5;
}

unsigned short PMS3003::getCF1PM10(){
	return PM_Data.CF1PM10;
}

unsigned short PMS3003::getPM1(){
	return PM_Data.PM1;
}

unsigned short PMS3003::getPM2dot5(){
	return PM_Data.PM2dot5;
}

unsigned short PMS3003::getPM10(){
	return PM_Data.PM10;
}

void PMS3003::PowerOff(){
	if(External_EN){
		//With external EN design
		SET->write(0); //High level for normal mode, low level for entering low power mode (Fan stopping)
		EN->write(0); //High level for power on, low level for power off (both logic part and fan sotpping)
	}else{
		//Without external EN design
		SET->write(0); //High level for normal mode, low level for entering low power mode (Fan stopping)
	}
}

void PMS3003::PowerOn(){
	if(External_EN){
		//With external EN design
		SET->write(1); //High level for normal mode, low level for entering low power mode (Fan stopping)
		EN->write(1); //High level for power on, low level for power off (both logic part and fan sotpping)
	}else{
		//Without external EN design
		SET->write(1); //High level for normal mode, low level for entering low power mode (Fan stopping)
	}
}

void PMS3003::Reset(){
		RESET->write(0); //Low level for reset
		wait_ms(10);    // Wait 10ms for reset
		RESET->write(1); //Low level for reset
}

bool PMS3003::isNormal(){
		return is_normal;
}
